@@ -26,7 +26,7 @@
 
 
 #define SCM_READER_VERSION_MAJOR 0
-#define SCM_READER_VERSION_MINOR 2
+#define SCM_READER_VERSION_MINOR 3
 
 #ifdef SCM_READER_USE_LIGHTNING
 
@@ -186,6 +186,14 @@ extern SCM scm_from_reader_spec (const scm_token_reader_spec_t *spec,
    ``escape'' its parent reader returning SCM_UNSPECIFIED.  */
 extern SCM scm_make_token_reader (SCM spec, SCM proc, SCM escape_p);
 
+
+/* The standard fault handler.  */
+extern SCM scm_reader_standard_fault_handler (SCM chr, SCM port, SCM reader);
+
+/* The `SCM' version (a Scheme procedure) of the above function.  */
+extern SCM scm_reader_standard_fault_handler_proc;
+
+
 
 /* Convenience macros for static initialization of C token readers.  */
 
@@ -198,7 +206,7 @@ extern SCM scm_make_token_reader (SCM spec, SCM proc, SCM escape_p);
     { .type = SCM_TOKEN_SINGLE, .value = { .single = (_chr) } },	\
     .name = (_name),							\
     .reader = { .type = SCM_TOKEN_READER_C,				\
-		.value.c_reader = (_func) },				\
+		.value = { .c_reader = (_func) } },			\
     .escape = _escape,							\
     .documentation = SCM_DEFTOKEN_MAKE_DOC (_doc)			\
   }
@@ -209,33 +217,31 @@ extern SCM scm_make_token_reader (SCM spec, SCM proc, SCM escape_p);
       .value = { .range = { .low = (_lo), .high = (_hi) } } },		\
     .name = (_name),							\
     .reader = { .type = SCM_TOKEN_READER_C,				\
-		.value.c_reader = (_func) },				\
+		.value = { .c_reader = (_func) } },			\
     .escape = _escape,							\
     .documentation = SCM_DEFTOKEN_MAKE_DOC (_doc)			\
   }
 
-#define SCM_DEFTOKEN_SET(_set, _name, _func, _escape, _doc)		\
-  {									\
-    { .type = SCM_TOKEN_SET, .value = { .set = (_set) } },		\
-    .name = (_name),							\
-    .reader = { .type = SCM_TOKEN_READER_C,				\
-	        .value.c_reader = (_func) },				\
-    .escape = _escape,							\
-    .documentation = SCM_DEFTOKEN_MAKE_DOC (_doc)		     	\
+#define SCM_DEFTOKEN_SET(_set, _name, _func, _escape, _doc)	\
+  {								\
+    { .type = SCM_TOKEN_SET, .value = { .set = (_set) } },	\
+    .name = (_name),						\
+    .reader = { .type = SCM_TOKEN_READER_C,			\
+	        .value = { .c_reader = (_func) } },		\
+    .escape = _escape,						\
+    .documentation = SCM_DEFTOKEN_MAKE_DOC (_doc)		\
   }
 
-#define SCM_END_TOKENS							  \
-  {									  \
-    { .type = SCM_TOKEN_UNDEF },					  \
-    .name = NULL,							  \
-    .reader = { .type = SCM_TOKEN_READER_UNDEF, .value.c_reader = NULL }, \
-    .documentation = NULL						  \
+#define SCM_END_TOKENS					\
+  {							\
+    { .type = SCM_TOKEN_UNDEF },			\
+    .name = NULL,					\
+    .reader = { .type = SCM_TOKEN_READER_UNDEF,		\
+		.value = { .c_reader = NULL } },	\
+    .documentation = NULL				\
   }
 
 
-/* A standard reader fault handler.  */
-extern SCM scm_reader_standard_fault_handler_proc;
-
 /* The SMOB type associated to `scm_reader_t', `scm_token_reader_spec_t', and
    `scm_token_reader_t'.  */
 extern scm_t_bits scm_reader_type, scm_token_reader_type,
