@@ -1,6 +1,6 @@
 /* A Scheme reader compiler for Guile.
 
-   Copyright (C) 2005  Ludovic Courts  <ludovic.courtes@laas.fr>
+   Copyright (C) 2005, 2006, 2007  Ludovic Courts  <ludovic.courtes@laas.fr>
 
    Part of the code here (a few `scm_token_reader_t' functions below) is
    based on Guile code released under the GNU LGPL (file `read.c') which
@@ -39,6 +39,7 @@
 #include "token-readers.h"
 
 #include "config.h"
+#include "compat.h"
 
 
 
@@ -144,8 +145,7 @@ scm_read_sexp (int chr, SCM port, scm_reader_t reader,
      symbol (see SCM_SYM_DOT below).  */
   register int c;
   register SCM tmp;
-  register SCM tl, tl2 = SCM_EOL;
-  SCM ans, ans2 = SCM_EOL;
+  register SCM tl, ans = SCM_EOL;
   int terminating_char = 0;
 
   switch (chr)
@@ -268,12 +268,14 @@ scm_read_string (int chr, SCM port, scm_reader_t scm_reader,
 		 scm_reader_t top_level_reader)
 #define FUNC_NAME "scm_read_string"
 {
-  SCM tok_buf;
-  char *str_buf;
+  /* For strings smaller than C_STR, this function creates only one Scheme
+     object (the string returned).  */
+
+  SCM str = SCM_BOOL_F;
+  char c_str[1024];
+  unsigned c_str_len = 0;
   int c;
-  unsigned j = 0;
 
-  tok_buf = scm_i_make_string (256, &str_buf);
   while ('"' != (c = scm_getc (port)))
     {
       if (c == EOF)
@@ -281,8 +283,19 @@ scm_read_string (int chr, SCM port, scm_reader_t scm_reader,
 				    "end of file in string constant",
 				    SCM_EOL);
 
-      while (j + 2 >= scm_i_string_length (tok_buf))
-	scm_grow_tok_buf (&tok_buf);
+      if (c_str_len + 1 >= sizeof (c_str))
+	{
+	  /* Flush the C buffer onto a Scheme string.  */
+	  SCM addy;
+
+	  if (str == SCM_BOOL_F)
+	    str = scm_c_make_string (0, SCM_MAKE_CHAR ('X'));
+
+	  addy = scm_from_locale_stringn (c_str, c_str_len);
+	  str = scm_string_append_shared (scm_list_2 (str, addy));
+
+	  c_str_len = 0;
+	}
 
       if (c == '\\')
 	switch (c = scm_getc (port))
@@ -292,7 +305,7 @@ scm_read_string (int chr, SCM port, scm_reader_t scm_reader,
 	  case '"':
 	  case '\\':
 	    break;
-#if SCM_ENABLE_ELISP
+#if 0 /* SCM_ENABLE_ELISP */
 	  case '(':
 	  case ')':
 	    if (SCM_ESCAPED_PARENS_P)
@@ -346,16 +359,23 @@ scm_read_string (int chr, SCM port, scm_reader_t scm_reader,
 			      "illegal character in escape sequence: ~S",
 			      scm_list_1 (SCM_MAKE_CHAR (c)));
 	  }
-      scm_c_string_set_x (tok_buf, j, SCM_MAKE_CHAR (c));
-      ++j;
+      c_str[c_str_len++] = c;
     }
-  if (j == 0)
-    return scm_nullstr;
 
-  /* Change this to scm_c_substring_read_only when
-     SCM_STRING_CHARS has been removed.
-  */
-  return scm_c_substring_copy (tok_buf, 0, j);
+  if (c_str_len > 0)
+    {
+      SCM addy;
+
+      addy = scm_from_locale_stringn (c_str, c_str_len);
+      if (str == SCM_BOOL_F)
+	str = addy;
+      else
+	str = scm_string_append_shared (scm_list_2 (str, addy));
+    }
+  else
+    str = (str == SCM_BOOL_F) ? scm_nullstr : str;
+
+  return str;
 }
 #undef FUNC_NAME
 
@@ -625,7 +645,7 @@ scm_read_character (int chr, SCM port, scm_reader_t reader,
        * does only consist of octal digits.  Finally, it should be
        * checked whether the resulting fixnum is in the range of
        * characters.  */
-      SCM p = scm_i_mem2number (charname, charname_len, 8);
+      SCM p = scm_c_locale_stringn_to_number (charname, charname_len, 8);
       if (SCM_I_INUMP (p))
 	return SCM_MAKE_CHAR (SCM_I_INUM (p));
     }
@@ -646,13 +666,22 @@ SCM
 scm_read_keyword (int chr, SCM port, scm_reader_t reader,
 		  scm_reader_t top_level_reader)
 {
+  SCM symbol;
+
   /* Invoke TOP_LEVEL_READER to read the symbol that comprises the keyword.
      Doing this instead of invoking a specific symbol reader function allows
      `scm_read_keyword ()' to adapt to the delimiters currently valid of
-     symbols.  */
-  return (scm_symbol_to_keyword (scm_call_reader (top_level_reader,
-						  port, 0,
-						  top_level_reader)));
+     symbols.
+
+     XXX: This implementation allows sloppy syntaxes like `#:  key' but so
+     does Guile's built-in reader.  */
+  symbol = scm_call_reader (top_level_reader, port, 0, top_level_reader);
+  if (!scm_is_symbol (symbol))
+    scm_i_input_error (__FUNCTION__, port,
+		       "symbol prefix `~a' not followed by a symbol: ~s",
+		       scm_list_2 (SCM_MAKE_CHAR (chr), symbol));
+
+  return (scm_symbol_to_keyword (symbol));
 }
 
 SCM
